(defpackage #:graph-matrix-test
  (:use :common-lisp
        :alexandria
        :metabang-bind
        :graph
        :graph-matrix
        :stefil
        :curry-compose-reader-macros)
  (:export :test))
